
/* **************************************************************************
  $Archive: /njcl_v2/src/com/novell/service/nds/net/NetCaseExactString.java $
  $Revision: 6 $
  $Modtime: 1/28/00 12:46p $
 
  Copyright (c) 1998 Novell, Inc.  All Rights Reserved.

  THIS WORK IS  SUBJECT  TO  U.S.  AND  INTERNATIONAL  COPYRIGHT  LAWS  AND
  TREATIES.   NO  PART  OF  THIS  WORK MAY BE  USED,  PRACTICED,  PERFORMED
  COPIED, DISTRIBUTED, REVISED, MODIFIED, TRANSLATED,  ABRIDGED, CONDENSED,
  EXPANDED,  COLLECTED,  COMPILED,  LINKED,  RECAST, TRANSFORMED OR ADAPTED
  WITHOUT THE PRIOR WRITTEN CONSENT OF NOVELL, INC. ANY USE OR EXPLOITATION
  OF THIS WORK WITHOUT AUTHORIZATION COULD SUBJECT THE PERPETRATOR TO
  CRIMINAL AND CIVIL LIABILITY.

****************************************************************************/

package com.novell.service.nds.net;


import java.io.*;

import com.novell.java.io.EndianInputStream;
import com.novell.java.io.EndianOutputStream;

import com.novell.service.jncp.NSIException;

import com.novell.service.nds.NdsCaseExactString;


/** @internal
 * NetCaseExactString is an Adapter for NdsCaseExactString.
 *
 * This class converts to/from the NWNet CE_String_T.
 *
 * API Data Structure:
 *    typedef pnchar CE_String_T;
 */
public final class NetCaseExactString
   extends NdsCaseExactString implements NetAttributeValue, Serializable
{
   /**
    * Constructs a Case Exact String from the "API Data Structure".
    *
    * @param   value   byte[] containing a CE_String_T.
    */
   protected NetCaseExactString (
         byte[] value)
      throws NSIException
   {
      try
      {
         EndianInputStream edis = new EndianInputStream (value);

         this.value = edis.readLoHiChars ();
      }
      catch (IOException e)
      {
         NSIException ne = new NSIException ();

         ne.setRootCause (e);
         throw (ne);
      }
   }

   /**
    * Constructs a NetCaseExactString adapter for an NdsCaseExactString.
    *
    * @param   value   an NdsCaseExactString object.
    */
   protected NetCaseExactString (
         NdsCaseExactString value)
   {
      super (value);
   }

   /**
    * Returns a Case Exact String "API Data Structure".
    *
    * @return   byte[] containing CE_String_T.
    */
   public byte[] toByte ()
      throws NSIException
   {
      try
      {
         ByteArrayOutputStream baos = new ByteArrayOutputStream ();
         EndianOutputStream eos = new EndianOutputStream (baos);

         eos.writeLoHiChars (this.value);
         return (baos.toByteArray ());
      }
      catch (IOException e)
      {
         NSIException ne = new NSIException ();

         ne.setRootCause (e);
         throw (ne);
      }
   }

} /* NetCaseExactString */


